			-*- Text -*-


One of the central concepts of this POS (peice of software) is that of a
packet.  A packet is a block of protocol data, which can be DEC-centric,
but need not be.  It has a number of properties, including:

*  Type, or encapsulation.  This can be one of the following:

   *  Ether: an ethernet frame.

   *  DDCMP: a DDCMP-encapsulated data block or a DDCMP control frame.

   *  RAW:   a raw data block.

*  Length, including framing.

*  Tag, an eight-bit number.

*  Protocol, like ANF-10, DECNET, LAT, IPv4, ...  This can be NONE if it
   is not known.

*  Node.  This is ANF-10 special.  More on that later.

Another concept is that of an object, which is something that can generate
packets based on external events, like traffic on an ethernet or incoming
data on a TCP or UDP connection.  It can also accept packets from other
objects depending on configuration.


A quick example, one end of an ethernet tunnel for DECnet:

  anftunnel -T tun -p 4711 -r 127.0.0.1 -E em0 -d -x tun

or as commands, either from a command file or given interactively:

  define tunnel tun port 4711 remote 127.0.0.1
  define ether em0 decnet xcon tun


Summary of command line arguments:

  anftunnel [options] [file] ...

    Help options:

     -h           Print this help text
     -v           Print version number

    The following options define objects:

     -B <name>    define bridge
     -C <name>    define remote command
     -E <name>    define ethernet
     -L <name>    define logger
     -M <name>    define multinet
     -S <name>    define sync/tcp
     -T <name>    define tunnel
     -U <name>    define sync/udp

    Global flags:

     -D           Disable command decoder
     -DD          Detach from terminal
     -V           Increase verbose level
     -Z           Show configuration and exit

    Args to configure objects:

     -4           sets IPv4 flag (ethernet)
     -6           sets IPv6 flag (ethernet)
     -a           sets ANF-10 flag (sync, ethernet)
     -c           sets CHAOS flag (ethernet)
     -d           sets DDCMP flag (sync)
     -d           sets DECnet flag (ethernet)
     -e           sets ethernet emulate flag (sync)
     -g <usecs>   sets inter-packet gap (IP connections)
     -m <number>  sets max node.  Strip neighbour msgs
     -i <name>    sets interface (ethernet)
     -k <key>     sets auth/encrypt key (tunnels, command)
     -l           sets LAT flag (ethernet)
     -l <port>    sets local port (sync/udp)
     -n <number>  sets node number for object
     -p <port>    sets port number (tunnels, sync, command)
     -p <proto>   sets protocol number (ethernet)
     -r           sets read-only mode (command)
     -r <addr>    do active connect (tunnel, sync/tcp)
     -r <addr>    sets remote addr (sync/udp)
     -s <bps>     sets max transmission speed (IP connections)
     -t <slot>    sets tag
     -v           sets object's verbose flag
     -x <name>    cross-connect to other object

    Non-option arguments are names of command files.


Interactive commands:

  clear object <obj> <parameter>
  clear <obj> <parameter>

    Clear the given parameter of the given object.  See the 'set'
    command for a list of arguments.

  clear verbose

    Clear the global verbose flag.

  define <type> <name> [optional parameters]

    Define (create) an object of the given type, with the given name.
    See the 'set' command for a list of parameters.  The new object
    will be placed in the stopped state, use the 'start' command to
    start it up.

  list objects

    List all objects, one per line.

  quit

    Quit a remote connection.  Has no effect on the local terminal.

  set object <obj> parameter [value]
  set <obj> parameter [value]

    Set the given parameter of the given object.

  set verbose

    Set global verbose flag.

  show ddcmp

    Show DDCMP status and details of all serial lines terminating it.

  show memory

    Show memory usage statistics.

  show object <obj>
  show <obj>

    Show information on the given object.  With no argument, show
    all objects.

  show version

    Show the current program version and edit level.

  start <obj>

    Start the given object, if it stopped.  With no argument, starts
    all stopped objects.

  stop <obj>

    Stop this object if it is running.


Object types:

  bridge

    This object is a bridge.  Objects that produce ethernet frames can
    be connected to it, and frame handling will be done based on MAC
    addresses.  There is no direct external connection.

  command

    This is not a packet handling object.  It implements a (remote)
    command decoder.  The basic parameters that can be set are a key
    to be used for authentication and encryption of the connection,
    and a read-only flag.  With no key specified normal telnet can be
    used for connecting to it.  Default port to listen to is 6016.

  ethernet

    This object connects to an ethernet interface.  Default interface
    is the name of the object.  Protocols to speak is specified with
    the various flags (anf-10, decnet, ...) with the ANF-10 protocol
    set by the protocol parameter, default is 0x6016.  If there is no
    protocol flag set, anf-10 is assumed.

  multinet

    This object talks multinet/tcp externally, and raw (decnet) frames
    on the inside.  

  sync

    This object talks DDCMP over TCP/IP.  If the ddcmp flag is set it
    will strip the DDCMP encapsulation and deliver raw frames, else it
    will pass the DDCMP frames on.  If the remote parameter is set, do
    an active connect to the port given, else listen for a connection
    on the given port.

  tunnel

    This object is a tunnel to somewhere else.  Packets can be of any
    type.  The tunnel is actually divided into sub-tunnels, all using
    different tags.

  udp

    This object is a version of the sync object that talks UDP/IP.
    The port and lport parameters need to be set.  If the remote
    parameter is not set, use localhost (127.0.0.1).

Object parameters:

  anf-10

    For an ethernet object, enble handling of anf-10 frames.  Default
    protocol type is 0x6016, but can be changed with the 'proto' param.

    For sync lines terminating DDCMP, enable the periodic transmission
    of REP frames that ANF-10 does.

  chaos

    For an ethernet object, enble handling of chaosnet frames.
    Protocol type is 0x0804.

  ddcmp

    For a sync line, terminate DDCMP here and send raw data on.

  decnet

    For an ethernet object, enble handling of DECnet frames.
    Protocol type is 0x6003.

  ethernet

    For a sync line, enable the ANF-10 sync-to-ether conversion magic.
    Implies 'ddcmp' and 'anf-10'.  If you cross-connect a sync line
    directly to an ethernet object, the ethernet flag is implied.

  interface

    For an ethernet object, set the interface to use.  Default is to
    use the object name.

  ipv4

    For an ethernet object, enble handling of IPv4 frames.
    Protocol type is 0x0800 and 0x0806 (arp).

  ipv6

    For an ethernet object, enble handling of DECnet frames.
    Protocol type is 0x86dd.

  key

    For tunnels and remote command handlers, set the key used for both
    encrypting and authentication of the remote part.

  lat

    For an ethernet object, enble handling of DECnet frames.
    Protocol type is 0x6004.

  lport <number>

    For a udp object, set the local source port to use.

  node

    For a sync or udp object doing sync-to-ether conversion, set the
    node to connect to on the ethernet side.

    For an ethernet object, limit connectivity to this node.

  port <number>

    For object talking TCP or UDP, set the port to listen to or connect
    to.

  proto <hex>

    For ethernet objects, set the ANF-10 ethernet protocol number to use.

  read-only

    For remote command objects, set the read-only flag.  This means that
    you can only view things, not change anything.

  remote <addr>

    For tunnels and sync/tcp objects, set remote address to use and do an
    active connect.  For sync/udp objects, set remote address to send
    packets to.

  tag

    (description needed)

  verbose

    Set the object's local verbose/debug flag.

  xcon <obj>

    Cross-connect this object to the other, i.e. send our outgoing frames
    to that object.  Depending on the types of the objects involved, set
    up a suitable connection back, if it makes sense.


ANF-10 ethernet to sync magic:

  (to be written)


Traffic shaping:

  Objects that communicate over TCP/IP can be configured to shape the
  traffic they generate.  This is done with an output packet queue and
  a separate thread handling the transmission.  The following parameters
  can be set:

  gap <microseconds>

    If non-zero this will wait at least the given number of microseconds
    between transmission of each packet.  Default value is zero.

  speed <bps>

    If non-zero this will compute a minimum gap to the next packet based
    on the given speed.  Default value is zero.

  queue <number of packets>

    Specify the maximum number of packets to keep in the output queue
    at a given time.  If this number is exceeded further packets will
    be dropped, until the output queue becomes empty.  Default value
    is ten (10) packets.
